@@ -6,6 +6,7 @@ module Hint.Base (
     InterpreterSession, SessionData(..), GhcErrLogger,
     InterpreterState(..), fromState, onState,
     InterpreterConfiguration(..),
+    ImportList(..), ModuleQualification(..), ModuleImport(..),
 
     runGhc1, runGhc2,
 
@@ -23,20 +24,17 @@ import Control.Monad.Catch as MC
 
 import Data.IORef
 import Data.Dynamic
+import qualified Data.List
 
 import qualified Hint.GHC as GHC
 
-#if MIN_VERSION_base(4,8,0)
-import qualified Data.List
-#endif
-
 import Hint.Extension
 
 -- | Version of the underlying ghc api. Values are:
 --
--- * @710@ for GHC 7.10.x
+-- * @802@ for GHC 8.2.x
 --
--- * @800@ for GHC 8.0.x
+-- * @804@ for GHC 8.4.x
 --
 -- * etc...
 ghcVersion :: Int
@@ -62,13 +60,28 @@ data InterpreterError = UnknownError String
 data InterpreterState = St {
                            activePhantoms    :: [PhantomModule],
                            zombiePhantoms    :: [PhantomModule],
+#if defined(NEED_PHANTOM_DIRECTORY)
+                           phantomDirectory  :: Maybe FilePath,
+#endif
                            hintSupportModule :: PhantomModule,
                            importQualHackMod :: Maybe PhantomModule,
-                           qualImports       :: [(ModuleName, String)],
+                           qualImports       :: [ModuleImport],
                            defaultExts       :: [(Extension, Bool)], -- R/O
                            configuration     :: InterpreterConfiguration
                         }
 
+data ImportList = NoImportList | ImportList [String] | HidingList [String]
+  deriving (Eq, Show)
+data ModuleQualification = NotQualified | ImportAs String | QualifiedAs (Maybe String)
+  deriving (Eq, Show)
+
+-- | Represent module import statement.
+--   See 'setImportsF'
+data ModuleImport = ModuleImport { modName :: String
+                                 , modQual :: ModuleQualification
+                                 , modImp  :: ImportList
+                                 } deriving (Show)
+
 data InterpreterConfiguration = Conf {
                                   searchFilePath :: [FilePath],
                                   languageExts   :: [Extension],
@@ -78,36 +91,22 @@ data InterpreterConfiguration = Conf {
 type InterpreterSession = SessionData ()
 
 instance Exception InterpreterError
-#if MIN_VERSION_base(4,8,0)
   where
     displayException (UnknownError err) = "UnknownError: " ++ err
     displayException (WontCompile  es)  = unlines . Data.List.nub . map errMsg $ es
     displayException (NotAllowed   err) = "NotAllowed: "   ++ err
     displayException (GhcException err) = "GhcException: " ++ err
-#endif
 
 type RunGhc  m a =
-#if __GLASGOW_HASKELL__ >= 800
     (forall n.(MonadIO n, MonadMask n) => GHC.GhcT n a)
-#else
-    (forall n.(MonadIO n, MonadMask n, Functor n) => GHC.GhcT n a)
-#endif
  -> m a
 
 type RunGhc1 m a b =
-#if __GLASGOW_HASKELL__ >= 800
     (forall n.(MonadIO n, MonadMask n) => a -> GHC.GhcT n b)
-#else
-    (forall n.(MonadIO n, MonadMask n, Functor n) => a -> GHC.GhcT n b)
-#endif
  -> (a -> m b)
 
 type RunGhc2 m a b c =
-#if __GLASGOW_HASKELL__ >= 800
     (forall n.(MonadIO n, MonadMask n) => a -> b -> GHC.GhcT n c)
-#else
-    (forall n.(MonadIO n, MonadMask n, Functor n) => a -> b -> GHC.GhcT n c)
-#endif
  -> (a -> b -> m c)
 
 data SessionData a = SessionData {
@@ -197,8 +196,9 @@ findModule mn = mapGhcExceptions NotAllowed $
 moduleIsLoaded :: MonadInterpreter m => ModuleName -> m Bool
 moduleIsLoaded mn = (findModule mn >> return True)
                    `catchIE` (\e -> case e of
-                                      NotAllowed{} -> return False
-                                      _            -> throwM e)
+                                      NotAllowed{}  -> return False
+                                      WontCompile{} -> return False
+                                      _             -> throwM e)
 
 withDynFlags :: MonadInterpreter m => (GHC.DynFlags -> m a) -> m a
 withDynFlags action
