@@ -1,7 +1,7 @@
 module Hint.Context (
       isModuleInterpreted,
       loadModules, getLoadedModules, setTopLevelModules,
-      setImports, setImportsQ,
+      setImports, setImportsQ, setImportsF,
       reset,
 
       PhantomModule(..),
@@ -13,6 +13,7 @@ module Hint.Context (
 import Prelude hiding (mod)
 
 import Data.Char
+import Data.Either (partitionEithers)
 import Data.List
 
 import Control.Arrow ((***))
@@ -23,7 +24,6 @@ import Control.Monad.Catch
 
 import Hint.Base
 import Hint.Conversions
-import qualified Hint.Util as Util
 import qualified Hint.CompatPlatform as Compat
 
 import qualified Hint.GHC as GHC
@@ -32,6 +32,12 @@ import System.Random
 import System.FilePath
 import System.Directory
 
+#if defined(NEED_PHANTOM_DIRECTORY)
+import Data.Maybe (maybe)
+import Hint.Configuration (setGhcOption)
+import System.IO.Temp
+#endif
+
 type ModuleText = String
 
 -- When creating a phantom module we have a situation similar to that of
@@ -50,20 +56,38 @@ newPhantomModule =
        let nums = concat [show (abs n::Int), show p, filter isDigit $ concat (ls ++ is)]
        let mod_name = 'M':nums
        --
-       tmp_dir <- liftIO getTemporaryDirectory
+       tmp_dir <- getPhantomDirectory
        --
-       return PhantomModule{pmName = mod_name, pmFile = tmp_dir </> nums}
+       return PhantomModule{pmName = mod_name, pmFile = tmp_dir </> mod_name <.> "hs"}
+
+getPhantomDirectory :: MonadInterpreter m => m FilePath
+getPhantomDirectory =
+#if defined(NEED_PHANTOM_DIRECTORY)
+    -- When a module is loaded by file name, ghc-8.4.1 loses track of the
+    -- file location after the first time it has been loaded, so we create
+    -- a directory for the phantom modules and add it to the search path.
+    do mfp <- fromState phantomDirectory
+       case mfp of
+           Just fp -> return fp
+           Nothing -> do tmp_dir <- liftIO getTemporaryDirectory
+                         fp <- liftIO $ createTempDirectory tmp_dir "hint"
+                         onState (\s -> s{ phantomDirectory = Just fp })
+                         setGhcOption $ "-i" ++ fp
+                         return fp
+#else
+    do liftIO getTemporaryDirectory
+#endif
 
 allModulesInContext :: MonadInterpreter m => m ([ModuleName], [ModuleName])
 allModulesInContext = runGhc getContextNames
 
-getContext :: GHC.GhcMonad m => m ([GHC.Module], [GHC.ImportDecl GHC.RdrName])
+getContext :: GHC.GhcMonad m => m ([GHC.Module], [GHC.ImportDecl GHC.GhcPs])
 getContext = GHC.getContext >>= foldM f ([], [])
   where
     f :: (GHC.GhcMonad m) =>
-         ([GHC.Module], [GHC.ImportDecl GHC.RdrName]) ->
+         ([GHC.Module], [GHC.ImportDecl GHC.GhcPs]) ->
          GHC.InteractiveImport ->
-         m ([GHC.Module], [GHC.ImportDecl GHC.RdrName])
+         m ([GHC.Module], [GHC.ImportDecl GHC.GhcPs])
     f (ns, ds) i = case i of
       (GHC.IIDecl d)     -> return (ns, d : ds)
       m@(GHC.IIModule _) -> do n <- iiModToMod m; return (n : ns, ds)
@@ -80,7 +104,7 @@ getContextNames = fmap (map name *** map decl) getContext
     where name = GHC.moduleNameString . GHC.moduleName
           decl = GHC.moduleNameString . GHC.unLoc . GHC.ideclName
 
-setContext :: GHC.GhcMonad m => [GHC.Module] -> [GHC.ImportDecl GHC.RdrName] -> m ()
+setContext :: GHC.GhcMonad m => [GHC.Module] -> [GHC.ImportDecl GHC.GhcPs] -> m ()
 setContext ms ds =
   let ms' = map modToIIMod ms
       ds' = map GHC.IIDecl ds
@@ -157,7 +181,7 @@ removePhantomModule pm =
        if safeToRemove
          then do mayFail $ do res <- runGhc1 GHC.load GHC.LoadAllTargets
                               return $ guard (isSucceeded res) >> Just ()
-                 liftIO $ removeFile (pmFile pm)
+              `finally` do liftIO $ removeFile (pmFile pm)
          else onState (\s -> s{zombiePhantoms = pm:zombiePhantoms s})
 
 -- Returns a tuple with the active and zombie phantom modules respectively
@@ -225,7 +249,8 @@ modNameFromSummary = moduleToString . GHC.ms_mod
 getLoadedModSummaries :: MonadInterpreter m => m [GHC.ModSummary]
 getLoadedModSummaries =
   do all_mod_summ <- runGhc GHC.getModuleGraph
-     filterM (runGhc1 GHC.isLoaded . GHC.ms_mod_name) all_mod_summ
+     filterM (runGhc1 GHC.isLoaded . GHC.ms_mod_name)
+             (GHC.mgModSummaries all_mod_summ)
 
 -- | Sets the modules whose context is used during evaluation. All bindings
 --   of these modules are in scope, not only those exported.
@@ -254,13 +279,13 @@ setTopLevelModules ms =
 
 -- | Sets the modules whose exports must be in context.
 --
---   Warning: 'setImports' and 'setImportsQ' are mutually exclusive.
+--   Warning: 'setImports', 'setImportsQ', and 'setImportsF' are mutually exclusive.
 --   If you have a list of modules to be used qualified and another list
 --   unqualified, then you need to do something like
 --
 --   >  setImportsQ ((zip unqualified $ repeat Nothing) ++ qualifieds)
 setImports :: MonadInterpreter m => [ModuleName] -> m ()
-setImports ms = setImportsQ $ zip ms (repeat Nothing)
+setImports ms = setImportsF $ map (\m -> ModuleImport m NotQualified NoImportList) ms
 
 -- | Sets the modules whose exports must be in context; some
 --   of them may be qualified. E.g.:
@@ -269,32 +294,52 @@ setImports ms = setImportsQ $ zip ms (repeat Nothing)
 --
 --   Here, "map" will refer to Prelude.map and "M.map" to Data.Map.map.
 setImportsQ :: MonadInterpreter m => [(ModuleName, Maybe String)] -> m ()
-setImportsQ ms =
-    do let qualOrNot (a, mb) = maybe (Right a) (Left . (,) a) mb
-           (quals, unquals) = Util.partitionEither $ map qualOrNot ms
-       --
-       unqual_mods <- mapM findModule unquals
-       mapM_ (findModule . fst) quals -- just to be sure they exist
+setImportsQ ms = setImportsF $ map (\(m,q) -> ModuleImport m (maybe NotQualified (QualifiedAs . Just) q) NoImportList) ms
+
+-- | Sets the modules whose exports must be in context; some
+--   may be qualified or have imports lists. E.g.:
+--
+--   @setImportsF [ModuleImport "Prelude" NotQualified NoImportList, ModuleImport "Data.Text" (QualifiedAs $ Just "Text") (HidingList ["pack"])]@
+
+setImportsF :: MonadInterpreter m => [ModuleImport] -> m ()
+setImportsF ms = do
+       regularMods <- mapM (findModule . modName) regularImports
+       mapM_ (findModule . modName) phantomImports -- just to be sure they exist
        --
        old_qual_hack_mod <- fromState importQualHackMod
        maybe (return ()) removePhantomModule old_qual_hack_mod
        --
-       new_pm <- if not $ null quals
-                   then do
+       new_pm <- if null phantomImports
+                   then return Nothing
+                   else do
                      new_pm <- addPhantomModule $ \mod_name -> unlines $
                                 ("module " ++ mod_name ++ " where ") :
-                                ["import qualified " ++ m ++ " as " ++ n |
-                                   (m,n) <- quals]
+                                map newImportLine phantomImports
                      onState (\s -> s{importQualHackMod = Just new_pm})
                      return $ Just new_pm
-                   else return Nothing
        --
        pm <- maybe (return []) (findModule . pmName >=> return . return) new_pm
        (old_top_level, _) <- runGhc getContext
        let new_top_level = pm ++ old_top_level
-       runGhc2 setContextModules new_top_level unqual_mods
+       runGhc2 setContextModules new_top_level regularMods
        --
-       onState (\s ->s{qualImports = quals})
+       onState (\s ->s{qualImports = phantomImports})
+  where
+    (regularImports, phantomImports) = partitionEithers $ map (\m -> if isQualified m || hasImportList m
+                                                                       then Right m
+                                                                       else Left m) ms
+    isQualified m = modQual m /= NotQualified
+    hasImportList m = modImp m /= NoImportList
+    newImportLine m = concat ["import ", case modQual m of
+                                            NotQualified -> modName m
+                                            ImportAs q -> modName m ++ " as " ++ q
+                                            QualifiedAs Nothing -> "qualified " ++ modName m
+                                            QualifiedAs (Just q) -> "qualified " ++ modName m ++ " as " ++ q
+                             ,case modImp m of
+                                 NoImportList -> ""
+                                 ImportList l -> " (" ++ intercalate "," l ++ ")"
+                                 HidingList l -> " hiding (" ++ intercalate "," l ++ ")"
+                             ]
 
 -- | 'cleanPhantomModules' works like 'reset', but skips the
 --   loading of the support module that installs '_show'. Its purpose
@@ -322,6 +367,11 @@ cleanPhantomModules =
                         importQualHackMod = Nothing,
                         qualImports         = []})
        liftIO $ mapM_ (removeFile . pmFile) (old_active ++ old_zombie)
+#if defined(NEED_PHANTOM_DIRECTORY)
+       old_phantomdir <- fromState phantomDirectory
+       onState (\s -> s{phantomDirectory    = Nothing})
+       liftIO $ do maybe (return ()) removeDirectory old_phantomdir
+#endif
 
 -- | All imported modules are cleared from the context, and
 --   loaded modules are unloaded. It is similar to a @:load@ in
